"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var node_path_1 = require("node:path");
/**
 * Locates module directories for given filenames by searching
 * the directory tree for package.json files.
 */
var ModuleDirectoryLocator = /** @class */ (function () {
    function ModuleDirectoryLocator(fileSystem, buildRoot, packageJsonReader) {
        this.fileSystem = fileSystem;
        this.buildRoot = buildRoot;
        this.packageJsonReader = packageJsonReader;
    }
    ModuleDirectoryLocator.prototype.getModuleDir = function (filename) {
        var moduleDir = filename.substring(0, filename.lastIndexOf(node_path_1.sep));
        return this.checkModuleDir(moduleDir);
    };
    ModuleDirectoryLocator.prototype.checkModuleDir = function (moduleDir) {
        var dirWithVersion = null;
        var dirWithLicense = null;
        var prevModuleDir = null;
        do {
            if (this.fileSystem.pathExists("".concat(moduleDir).concat(node_path_1.sep, "package.json"))) {
                var packageMeta = this.packageJsonReader.readPackageJson(moduleDir);
                if (packageMeta.name !== undefined &&
                    packageMeta.version !== undefined) {
                    dirWithVersion = moduleDir;
                    if (packageMeta.license !== undefined ||
                        packageMeta.licenses !== undefined) {
                        dirWithLicense = moduleDir;
                    }
                }
            }
            prevModuleDir = moduleDir;
            moduleDir = (0, node_path_1.resolve)("".concat(moduleDir).concat(node_path_1.sep, "..").concat(node_path_1.sep));
        } while (!dirWithLicense &&
            moduleDir !== prevModuleDir &&
            moduleDir !== this.buildRoot);
        return dirWithLicense || dirWithVersion;
    };
    return ModuleDirectoryLocator;
}());
exports.default = ModuleDirectoryLocator;
//# sourceMappingURL=ModuleDirectoryLocator.js.map